// ---------------------------------------------------------------------------
// Eine Wechselspannungsquelle (50 Hz, 15 Volt Effektivwert) mit einem
// Innenwiderstand von 5 Ohm versorgt über einen Brückengleichrichter mit
// einem Glättungskondensator von 470 µF einen Lastwiderstand von 15 Ohm.
// Die Durchlassspannung der Gleichrichterdioden beträgt jeweils 0,75 Volt.
//
// 2018-05-10, Tilman Küpper (tilman.kuepper@hm.edu)
// ---------------------------------------------------------------------------

#include "chart.h"  // Darstellung grafischer Kurvenverläufe
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

double dua_dt(double ua, double t);
void euler(double t_start, double t_end, double t_step, double ua_start);

#define U0 15.0    // Effektivwert Wechselspannungsquelle
#define RI 5.0     // Innenwiderstand Spannungsquelle
#define FR 50.0    // Frequenz in Hz
#define UF 0.75    // Durchlassspannung einer Diode
#define CG 470e-6  // Kapazität Glättungskondensator
#define RA 15.0    // Lastwiderstand

// Ausgangsspannung zu Beginn der Simulation
#define UA_START 0.0

// Zeitintervall für Simulation
#define T_START 0.000
#define T_STOP  0.025
#define T_DELTA 0.000005

// ---------------------------------------------------------------------------
// Hauptprogramm.
// ---------------------------------------------------------------------------
int main(void)
{
    euler(T_START, T_STOP, T_DELTA, UA_START);
    return 0;
}

// ---------------------------------------------------------------------------
// Der Spannungsverlauf am Lastwiderstand wird durch eine DGL 1. Ordnung
// beschrieben. Zur Lösung dieser DGL wird das Eulerverfahren verwendet.
// ---------------------------------------------------------------------------
void euler(double t_start, double t_end, double t_step, double ua_start)
{
    double t = t_start, ua = ua_start;
    chart_moveto(t_start, ua_start);

    // Eulerverfahren, bis t_end erreicht ist
    while(t < t_end)
    {
        ua += t_step * dua_dt(ua, t);
        t  += t_step;
        chart_lineto(t, ua, CHART_RED);
    }

    // Grafische Darstellung des Spannungsverlaufs
    chart_show(CHART_DEFAULT, "Gleichrichter");
}

// ---------------------------------------------------------------------------
// Der Spannungsverlauf am Lastwiderstand wird durch die hier angegebene
// Differentialgleichung 1. Ordnung beschrieben:
//
// Falls |u0(t)| - 2 * Uf > ua(t):
//     dua(t)/dt = (|u0(t)| - ua(t) - 2 * Uf) / (Ri * Cg) - ua(t) / (Ra * Cg)
//
// Ansonsten:
//     dua(t)/dt = -ua(t) / (Ra * Cg)
//
// Dabei ist u0(t) die Wechselspannung am Eingang des Brückengleichrichters.
// ---------------------------------------------------------------------------
double dua_dt(double ua, double t)
{
    double ampl  = U0 * sqrt(2.0);
    double omega = 2.0 * M_PI * FR;
    double u0    = ampl * sin(omega * t);

    if(fabs(u0) - 2 * UF > ua)
        return (fabs(u0) - ua - 2 * UF) / (RI * CG) - ua / (RA * CG);
    else
        return -ua / (RA * CG);
}
